# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
# This file is machine-generated by lib/unicore/mktables from the Unicode
# database, Version 15.0.0.  Any changes made here will be lost!

# !!!!!!!   INTERNAL PERL USE ONLY   !!!!!!!
# This file is for internal use by core Perl only.  The format and even the
# name or existence of this file are subject to change without notice.  Don't
# use it directly.  Use Unicode::UCD to access the Unicode character data
# base.


return <<'END';
V28
9001
9003
12296
12306
12308
12320
12336
12337
12448
12449
12540
12541
65113
65119
65288
65290
65306
65308
65339
65340
65341
65342
65343
65344
65371
65377
65507
65508
END
