# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
# This file is machine-generated by lib/unicore/mktables from the Unicode
# database, Version 15.0.0.  Any changes made here will be lost!

# !!!!!!!   INTERNAL PERL USE ONLY   !!!!!!!
# This file is for internal use by core Perl only.  The format and even the
# name or existence of this file are subject to change without notice.  Don't
# use it directly.  Use Unicode::UCD to access the Unicode character data
# base.


return <<'END';
V34
173
174
847
848
1564
1565
4447
4449
6068
6070
6155
6160
8203
8208
8234
8239
8288
8304
12644
12645
65024
65040
65279
65280
65440
65441
65520
65529
113824
113828
119155
119163
917504
921600
END
