# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
# This file is machine-generated by lib/unicore/mktables from the Unicode
# database, Version 15.0.0.  Any changes made here will be lost!

# !!!!!!!   INTERNAL PERL USE ONLY   !!!!!!!
# This file is for internal use by core Perl only.  The format and even the
# name or existence of this file are subject to change without notice.  Don't
# use it directly.  Use Unicode::UCD to access the Unicode character data
# base.



# The name this table is to be known by, with the format of the mappings in
# the main body of the table, and what all code points missing from this file
# map to.
$Unicode::UCD::SwashInfo{'ToGCB'}{'format'} = 's'; # string
$Unicode::UCD::SwashInfo{'ToGCB'}{'missing'} = 'Other';

return <<'END';
0	9	Control
A		LF
B	C	Control
D		CR
E	1F	Control
7F	9F	Control
A9		ExtPict_XX
AD		Control
AE		ExtPict_XX
300	36F	Extend
483	489	Extend
591	5BD	Extend
5BF		Extend
5C1	5C2	Extend
5C4	5C5	Extend
5C7		Extend
600	605	Prepend
610	61A	Extend
61C		Control
64B	65F	Extend
670		Extend
6D6	6DC	Extend
6DD		Prepend
6DF	6E4	Extend
6E7	6E8	Extend
6EA	6ED	Extend
70F		Prepend
711		Extend
730	74A	Extend
7A6	7B0	Extend
7EB	7F3	Extend
7FD		Extend
816	819	Extend
81B	823	Extend
825	827	Extend
829	82D	Extend
859	85B	Extend
890	891	Prepend
898	89F	Extend
8CA	8E1	Extend
8E2		Prepend
8E3	902	Extend
903		SpacingMark
93A		Extend
93B		SpacingMark
93C		Extend
93E	940	SpacingMark
941	948	Extend
949	94C	SpacingMark
94D		Extend
94E	94F	SpacingMark
951	957	Extend
962	963	Extend
981		Extend
982	983	SpacingMark
9BC		Extend
9BE		Extend
9BF	9C0	SpacingMark
9C1	9C4	Extend
9C7	9C8	SpacingMark
9CB	9CC	SpacingMark
9CD		Extend
9D7		Extend
9E2	9E3	Extend
9FE		Extend
A01	A02	Extend
A03		SpacingMark
A3C		Extend
A3E	A40	SpacingMark
A41	A42	Extend
A47	A48	Extend
A4B	A4D	Extend
A51		Extend
A70	A71	Extend
A75		Extend
A81	A82	Extend
A83		SpacingMark
ABC		Extend
ABE	AC0	SpacingMark
AC1	AC5	Extend
AC7	AC8	Extend
AC9		SpacingMark
ACB	ACC	SpacingMark
ACD		Extend
AE2	AE3	Extend
AFA	AFF	Extend
B01		Extend
B02	B03	SpacingMark
B3C		Extend
B3E	B3F	Extend
B40		SpacingMark
B41	B44	Extend
B47	B48	SpacingMark
B4B	B4C	SpacingMark
B4D		Extend
B55	B57	Extend
B62	B63	Extend
B82		Extend
BBE		Extend
BBF		SpacingMark
BC0		Extend
BC1	BC2	SpacingMark
BC6	BC8	SpacingMark
BCA	BCC	SpacingMark
BCD		Extend
BD7		Extend
C00		Extend
C01	C03	SpacingMark
C04		Extend
C3C		Extend
C3E	C40	Extend
C41	C44	SpacingMark
C46	C48	Extend
C4A	C4D	Extend
C55	C56	Extend
C62	C63	Extend
C81		Extend
C82	C83	SpacingMark
CBC		Extend
CBE		SpacingMark
CBF		Extend
CC0	CC1	SpacingMark
CC2		Extend
CC3	CC4	SpacingMark
CC6		Extend
CC7	CC8	SpacingMark
CCA	CCB	SpacingMark
CCC	CCD	Extend
CD5	CD6	Extend
CE2	CE3	Extend
CF3		SpacingMark
D00	D01	Extend
D02	D03	SpacingMark
D3B	D3C	Extend
D3E		Extend
D3F	D40	SpacingMark
D41	D44	Extend
D46	D48	SpacingMark
D4A	D4C	SpacingMark
D4D		Extend
D4E		Prepend
D57		Extend
D62	D63	Extend
D81		Extend
D82	D83	SpacingMark
DCA		Extend
DCF		Extend
DD0	DD1	SpacingMark
DD2	DD4	Extend
DD6		Extend
DD8	DDE	SpacingMark
DDF		Extend
DF2	DF3	SpacingMark
E31		Extend
E33		SpacingMark
E34	E3A	Extend
E47	E4E	Extend
EB1		Extend
EB3		SpacingMark
EB4	EBC	Extend
EC8	ECE	Extend
F18	F19	Extend
F35		Extend
F37		Extend
F39		Extend
F3E	F3F	SpacingMark
F71	F7E	Extend
F7F		SpacingMark
F80	F84	Extend
F86	F87	Extend
F8D	F97	Extend
F99	FBC	Extend
FC6		Extend
102D	1030	Extend
1031		SpacingMark
1032	1037	Extend
1039	103A	Extend
103B	103C	SpacingMark
103D	103E	Extend
1056	1057	SpacingMark
1058	1059	Extend
105E	1060	Extend
1071	1074	Extend
1082		Extend
1084		SpacingMark
1085	1086	Extend
108D		Extend
109D		Extend
1100	115F	L
1160	11A7	V
11A8	11FF	T
135D	135F	Extend
1712	1714	Extend
1715		SpacingMark
1732	1733	Extend
1734		SpacingMark
1752	1753	Extend
1772	1773	Extend
17B4	17B5	Extend
17B6		SpacingMark
17B7	17BD	Extend
17BE	17C5	SpacingMark
17C6		Extend
17C7	17C8	SpacingMark
17C9	17D3	Extend
17DD		Extend
180B	180D	Extend
180E		Control
180F		Extend
1885	1886	Extend
18A9		Extend
1920	1922	Extend
1923	1926	SpacingMark
1927	1928	Extend
1929	192B	SpacingMark
1930	1931	SpacingMark
1932		Extend
1933	1938	SpacingMark
1939	193B	Extend
1A17	1A18	Extend
1A19	1A1A	SpacingMark
1A1B		Extend
1A55		SpacingMark
1A56		Extend
1A57		SpacingMark
1A58	1A5E	Extend
1A60		Extend
1A62		Extend
1A65	1A6C	Extend
1A6D	1A72	SpacingMark
1A73	1A7C	Extend
1A7F		Extend
1AB0	1ACE	Extend
1B00	1B03	Extend
1B04		SpacingMark
1B34	1B3A	Extend
1B3B		SpacingMark
1B3C		Extend
1B3D	1B41	SpacingMark
1B42		Extend
1B43	1B44	SpacingMark
1B6B	1B73	Extend
1B80	1B81	Extend
1B82		SpacingMark
1BA1		SpacingMark
1BA2	1BA5	Extend
1BA6	1BA7	SpacingMark
1BA8	1BA9	Extend
1BAA		SpacingMark
1BAB	1BAD	Extend
1BE6		Extend
1BE7		SpacingMark
1BE8	1BE9	Extend
1BEA	1BEC	SpacingMark
1BED		Extend
1BEE		SpacingMark
1BEF	1BF1	Extend
1BF2	1BF3	SpacingMark
1C24	1C2B	SpacingMark
1C2C	1C33	Extend
1C34	1C35	SpacingMark
1C36	1C37	Extend
1CD0	1CD2	Extend
1CD4	1CE0	Extend
1CE1		SpacingMark
1CE2	1CE8	Extend
1CED		Extend
1CF4		Extend
1CF7		SpacingMark
1CF8	1CF9	Extend
1DC0	1DFF	Extend
200B		Control
200C		Extend
200D		ZWJ
200E	200F	Control
2028	202E	Control
203C		ExtPict_XX
2049		ExtPict_XX
2060	206F	Control
20D0	20F0	Extend
2122		ExtPict_XX
2139		ExtPict_XX
2194	2199	ExtPict_XX
21A9	21AA	ExtPict_XX
231A	231B	ExtPict_XX
2328		ExtPict_XX
2388		ExtPict_XX
23CF		ExtPict_XX
23E9	23F3	ExtPict_XX
23F8	23FA	ExtPict_XX
24C2		ExtPict_XX
25AA	25AB	ExtPict_XX
25B6		ExtPict_XX
25C0		ExtPict_XX
25FB	25FE	ExtPict_XX
2600	2605	ExtPict_XX
2607	2612	ExtPict_XX
2614	2685	ExtPict_XX
2690	2705	ExtPict_XX
2708	2712	ExtPict_XX
2714		ExtPict_XX
2716		ExtPict_XX
271D		ExtPict_XX
2721		ExtPict_XX
2728		ExtPict_XX
2733	2734	ExtPict_XX
2744		ExtPict_XX
2747		ExtPict_XX
274C		ExtPict_XX
274E		ExtPict_XX
2753	2755	ExtPict_XX
2757		ExtPict_XX
2763	2767	ExtPict_XX
2795	2797	ExtPict_XX
27A1		ExtPict_XX
27B0		ExtPict_XX
27BF		ExtPict_XX
2934	2935	ExtPict_XX
2B05	2B07	ExtPict_XX
2B1B	2B1C	ExtPict_XX
2B50		ExtPict_XX
2B55		ExtPict_XX
2CEF	2CF1	Extend
2D7F		Extend
2DE0	2DFF	Extend
302A	302F	Extend
3030		ExtPict_XX
303D		ExtPict_XX
3099	309A	Extend
3297		ExtPict_XX
3299		ExtPict_XX
A66F	A672	Extend
A674	A67D	Extend
A69E	A69F	Extend
A6F0	A6F1	Extend
A802		Extend
A806		Extend
A80B		Extend
A823	A824	SpacingMark
A825	A826	Extend
A827		SpacingMark
A82C		Extend
A880	A881	SpacingMark
A8B4	A8C3	SpacingMark
A8C4	A8C5	Extend
A8E0	A8F1	Extend
A8FF		Extend
A926	A92D	Extend
A947	A951	Extend
A952	A953	SpacingMark
A960	A97C	L
A980	A982	Extend
A983		SpacingMark
A9B3		Extend
A9B4	A9B5	SpacingMark
A9B6	A9B9	Extend
A9BA	A9BB	SpacingMark
A9BC	A9BD	Extend
A9BE	A9C0	SpacingMark
A9E5		Extend
AA29	AA2E	Extend
AA2F	AA30	SpacingMark
AA31	AA32	Extend
AA33	AA34	SpacingMark
AA35	AA36	Extend
AA43		Extend
AA4C		Extend
AA4D		SpacingMark
AA7C		Extend
AAB0		Extend
AAB2	AAB4	Extend
AAB7	AAB8	Extend
AABE	AABF	Extend
AAC1		Extend
AAEB		SpacingMark
AAEC	AAED	Extend
AAEE	AAEF	SpacingMark
AAF5		SpacingMark
AAF6		Extend
ABE3	ABE4	SpacingMark
ABE5		Extend
ABE6	ABE7	SpacingMark
ABE8		Extend
ABE9	ABEA	SpacingMark
ABEC		SpacingMark
ABED		Extend
AC00		LV
AC01	AC1B	LVT
AC1C		LV
AC1D	AC37	LVT
AC38		LV
AC39	AC53	LVT
AC54		LV
AC55	AC6F	LVT
AC70		LV
AC71	AC8B	LVT
AC8C		LV
AC8D	ACA7	LVT
ACA8		LV
ACA9	ACC3	LVT
ACC4		LV
ACC5	ACDF	LVT
ACE0		LV
ACE1	ACFB	LVT
ACFC		LV
ACFD	AD17	LVT
AD18		LV
AD19	AD33	LVT
AD34		LV
AD35	AD4F	LVT
AD50		LV
AD51	AD6B	LVT
AD6C		LV
AD6D	AD87	LVT
AD88		LV
AD89	ADA3	LVT
ADA4		LV
ADA5	ADBF	LVT
ADC0		LV
ADC1	ADDB	LVT
ADDC		LV
ADDD	ADF7	LVT
ADF8		LV
ADF9	AE13	LVT
AE14		LV
AE15	AE2F	LVT
AE30		LV
AE31	AE4B	LVT
AE4C		LV
AE4D	AE67	LVT
AE68		LV
AE69	AE83	LVT
AE84		LV
AE85	AE9F	LVT
AEA0		LV
AEA1	AEBB	LVT
AEBC		LV
AEBD	AED7	LVT
AED8		LV
AED9	AEF3	LVT
AEF4		LV
AEF5	AF0F	LVT
AF10		LV
AF11	AF2B	LVT
AF2C		LV
AF2D	AF47	LVT
AF48		LV
AF49	AF63	LVT
AF64		LV
AF65	AF7F	LVT
AF80		LV
AF81	AF9B	LVT
AF9C		LV
AF9D	AFB7	LVT
AFB8		LV
AFB9	AFD3	LVT
AFD4		LV
AFD5	AFEF	LVT
AFF0		LV
AFF1	B00B	LVT
B00C		LV
B00D	B027	LVT
B028		LV
B029	B043	LVT
B044		LV
B045	B05F	LVT
B060		LV
B061	B07B	LVT
B07C		LV
B07D	B097	LVT
B098		LV
B099	B0B3	LVT
B0B4		LV
B0B5	B0CF	LVT
B0D0		LV
B0D1	B0EB	LVT
B0EC		LV
B0ED	B107	LVT
B108		LV
B109	B123	LVT
B124		LV
B125	B13F	LVT
B140		LV
B141	B15B	LVT
B15C		LV
B15D	B177	LVT
B178		LV
B179	B193	LVT
B194		LV
B195	B1AF	LVT
B1B0		LV
B1B1	B1CB	LVT
B1CC		LV
B1CD	B1E7	LVT
B1E8		LV
B1E9	B203	LVT
B204		LV
B205	B21F	LVT
B220		LV
B221	B23B	LVT
B23C		LV
B23D	B257	LVT
B258		LV
B259	B273	LVT
B274		LV
B275	B28F	LVT
B290		LV
B291	B2AB	LVT
B2AC		LV
B2AD	B2C7	LVT
B2C8		LV
B2C9	B2E3	LVT
B2E4		LV
B2E5	B2FF	LVT
B300		LV
B301	B31B	LVT
B31C		LV
B31D	B337	LVT
B338		LV
B339	B353	LVT
B354		LV
B355	B36F	LVT
B370		LV
B371	B38B	LVT
B38C		LV
B38D	B3A7	LVT
B3A8		LV
B3A9	B3C3	LVT
B3C4		LV
B3C5	B3DF	LVT
B3E0		LV
B3E1	B3FB	LVT
B3FC		LV
B3FD	B417	LVT
B418		LV
B419	B433	LVT
B434		LV
B435	B44F	LVT
B450		LV
B451	B46B	LVT
B46C		LV
B46D	B487	LVT
B488		LV
B489	B4A3	LVT
B4A4		LV
B4A5	B4BF	LVT
B4C0		LV
B4C1	B4DB	LVT
B4DC		LV
B4DD	B4F7	LVT
B4F8		LV
B4F9	B513	LVT
B514		LV
B515	B52F	LVT
B530		LV
B531	B54B	LVT
B54C		LV
B54D	B567	LVT
B568		LV
B569	B583	LVT
B584		LV
B585	B59F	LVT
B5A0		LV
B5A1	B5BB	LVT
B5BC		LV
B5BD	B5D7	LVT
B5D8		LV
B5D9	B5F3	LVT
B5F4		LV
B5F5	B60F	LVT
B610		LV
B611	B62B	LVT
B62C		LV
B62D	B647	LVT
B648		LV
B649	B663	LVT
B664		LV
B665	B67F	LVT
B680		LV
B681	B69B	LVT
B69C		LV
B69D	B6B7	LVT
B6B8		LV
B6B9	B6D3	LVT
B6D4		LV
B6D5	B6EF	LVT
B6F0		LV
B6F1	B70B	LVT
B70C		LV
B70D	B727	LVT
B728		LV
B729	B743	LVT
B744		LV
B745	B75F	LVT
B760		LV
B761	B77B	LVT
B77C		LV
B77D	B797	LVT
B798		LV
B799	B7B3	LVT
B7B4		LV
B7B5	B7CF	LVT
B7D0		LV
B7D1	B7EB	LVT
B7EC		LV
B7ED	B807	LVT
B808		LV
B809	B823	LVT
B824		LV
B825	B83F	LVT
B840		LV
B841	B85B	LVT
B85C		LV
B85D	B877	LVT
B878		LV
B879	B893	LVT
B894		LV
B895	B8AF	LVT
B8B0		LV
B8B1	B8CB	LVT
B8CC		LV
B8CD	B8E7	LVT
B8E8		LV
B8E9	B903	LVT
B904		LV
B905	B91F	LVT
B920		LV
B921	B93B	LVT
B93C		LV
B93D	B957	LVT
B958		LV
B959	B973	LVT
B974		LV
B975	B98F	LVT
B990		LV
B991	B9AB	LVT
B9AC		LV
B9AD	B9C7	LVT
B9C8		LV
B9C9	B9E3	LVT
B9E4		LV
B9E5	B9FF	LVT
BA00		LV
BA01	BA1B	LVT
BA1C		LV
BA1D	BA37	LVT
BA38		LV
BA39	BA53	LVT
BA54		LV
BA55	BA6F	LVT
BA70		LV
BA71	BA8B	LVT
BA8C		LV
BA8D	BAA7	LVT
BAA8		LV
BAA9	BAC3	LVT
BAC4		LV
BAC5	BADF	LVT
BAE0		LV
BAE1	BAFB	LVT
BAFC		LV
BAFD	BB17	LVT
BB18		LV
BB19	BB33	LVT
BB34		LV
BB35	BB4F	LVT
BB50		LV
BB51	BB6B	LVT
BB6C		LV
BB6D	BB87	LVT
BB88		LV
BB89	BBA3	LVT
BBA4		LV
BBA5	BBBF	LVT
BBC0		LV
BBC1	BBDB	LVT
BBDC		LV
BBDD	BBF7	LVT
BBF8		LV
BBF9	BC13	LVT
BC14		LV
BC15	BC2F	LVT
BC30		LV
BC31	BC4B	LVT
BC4C		LV
BC4D	BC67	LVT
BC68		LV
BC69	BC83	LVT
BC84		LV
BC85	BC9F	LVT
BCA0		LV
BCA1	BCBB	LVT
BCBC		LV
BCBD	BCD7	LVT
BCD8		LV
BCD9	BCF3	LVT
BCF4		LV
BCF5	BD0F	LVT
BD10		LV
BD11	BD2B	LVT
BD2C		LV
BD2D	BD47	LVT
BD48		LV
BD49	BD63	LVT
BD64		LV
BD65	BD7F	LVT
BD80		LV
BD81	BD9B	LVT
BD9C		LV
BD9D	BDB7	LVT
BDB8		LV
BDB9	BDD3	LVT
BDD4		LV
BDD5	BDEF	LVT
BDF0		LV
BDF1	BE0B	LVT
BE0C		LV
BE0D	BE27	LVT
BE28		LV
BE29	BE43	LVT
BE44		LV
BE45	BE5F	LVT
BE60		LV
BE61	BE7B	LVT
BE7C		LV
BE7D	BE97	LVT
BE98		LV
BE99	BEB3	LVT
BEB4		LV
BEB5	BECF	LVT
BED0		LV
BED1	BEEB	LVT
BEEC		LV
BEED	BF07	LVT
BF08		LV
BF09	BF23	LVT
BF24		LV
BF25	BF3F	LVT
BF40		LV
BF41	BF5B	LVT
BF5C		LV
BF5D	BF77	LVT
BF78		LV
BF79	BF93	LVT
BF94		LV
BF95	BFAF	LVT
BFB0		LV
BFB1	BFCB	LVT
BFCC		LV
BFCD	BFE7	LVT
BFE8		LV
BFE9	C003	LVT
C004		LV
C005	C01F	LVT
C020		LV
C021	C03B	LVT
C03C		LV
C03D	C057	LVT
C058		LV
C059	C073	LVT
C074		LV
C075	C08F	LVT
C090		LV
C091	C0AB	LVT
C0AC		LV
C0AD	C0C7	LVT
C0C8		LV
C0C9	C0E3	LVT
C0E4		LV
C0E5	C0FF	LVT
C100		LV
C101	C11B	LVT
C11C		LV
C11D	C137	LVT
C138		LV
C139	C153	LVT
C154		LV
C155	C16F	LVT
C170		LV
C171	C18B	LVT
C18C		LV
C18D	C1A7	LVT
C1A8		LV
C1A9	C1C3	LVT
C1C4		LV
C1C5	C1DF	LVT
C1E0		LV
C1E1	C1FB	LVT
C1FC		LV
C1FD	C217	LVT
C218		LV
C219	C233	LVT
C234		LV
C235	C24F	LVT
C250		LV
C251	C26B	LVT
C26C		LV
C26D	C287	LVT
C288		LV
C289	C2A3	LVT
C2A4		LV
C2A5	C2BF	LVT
C2C0		LV
C2C1	C2DB	LVT
C2DC		LV
C2DD	C2F7	LVT
C2F8		LV
C2F9	C313	LVT
C314		LV
C315	C32F	LVT
C330		LV
C331	C34B	LVT
C34C		LV
C34D	C367	LVT
C368		LV
C369	C383	LVT
C384		LV
C385	C39F	LVT
C3A0		LV
C3A1	C3BB	LVT
C3BC		LV
C3BD	C3D7	LVT
C3D8		LV
C3D9	C3F3	LVT
C3F4		LV
C3F5	C40F	LVT
C410		LV
C411	C42B	LVT
C42C		LV
C42D	C447	LVT
C448		LV
C449	C463	LVT
C464		LV
C465	C47F	LVT
C480		LV
C481	C49B	LVT
C49C		LV
C49D	C4B7	LVT
C4B8		LV
C4B9	C4D3	LVT
C4D4		LV
C4D5	C4EF	LVT
C4F0		LV
C4F1	C50B	LVT
C50C		LV
C50D	C527	LVT
C528		LV
C529	C543	LVT
C544		LV
C545	C55F	LVT
C560		LV
C561	C57B	LVT
C57C		LV
C57D	C597	LVT
C598		LV
C599	C5B3	LVT
C5B4		LV
C5B5	C5CF	LVT
C5D0		LV
C5D1	C5EB	LVT
C5EC		LV
C5ED	C607	LVT
C608		LV
C609	C623	LVT
C624		LV
C625	C63F	LVT
C640		LV
C641	C65B	LVT
C65C		LV
C65D	C677	LVT
C678		LV
C679	C693	LVT
C694		LV
C695	C6AF	LVT
C6B0		LV
C6B1	C6CB	LVT
C6CC		LV
C6CD	C6E7	LVT
C6E8		LV
C6E9	C703	LVT
C704		LV
C705	C71F	LVT
C720		LV
C721	C73B	LVT
C73C		LV
C73D	C757	LVT
C758		LV
C759	C773	LVT
C774		LV
C775	C78F	LVT
C790		LV
C791	C7AB	LVT
C7AC		LV
C7AD	C7C7	LVT
C7C8		LV
C7C9	C7E3	LVT
C7E4		LV
C7E5	C7FF	LVT
C800		LV
C801	C81B	LVT
C81C		LV
C81D	C837	LVT
C838		LV
C839	C853	LVT
C854		LV
C855	C86F	LVT
C870		LV
C871	C88B	LVT
C88C		LV
C88D	C8A7	LVT
C8A8		LV
C8A9	C8C3	LVT
C8C4		LV
C8C5	C8DF	LVT
C8E0		LV
C8E1	C8FB	LVT
C8FC		LV
C8FD	C917	LVT
C918		LV
C919	C933	LVT
C934		LV
C935	C94F	LVT
C950		LV
C951	C96B	LVT
C96C		LV
C96D	C987	LVT
C988		LV
C989	C9A3	LVT
C9A4		LV
C9A5	C9BF	LVT
C9C0		LV
C9C1	C9DB	LVT
C9DC		LV
C9DD	C9F7	LVT
C9F8		LV
C9F9	CA13	LVT
CA14		LV
CA15	CA2F	LVT
CA30		LV
CA31	CA4B	LVT
CA4C		LV
CA4D	CA67	LVT
CA68		LV
CA69	CA83	LVT
CA84		LV
CA85	CA9F	LVT
CAA0		LV
CAA1	CABB	LVT
CABC		LV
CABD	CAD7	LVT
CAD8		LV
CAD9	CAF3	LVT
CAF4		LV
CAF5	CB0F	LVT
CB10		LV
CB11	CB2B	LVT
CB2C		LV
CB2D	CB47	LVT
CB48		LV
CB49	CB63	LVT
CB64		LV
CB65	CB7F	LVT
CB80		LV
CB81	CB9B	LVT
CB9C		LV
CB9D	CBB7	LVT
CBB8		LV
CBB9	CBD3	LVT
CBD4		LV
CBD5	CBEF	LVT
CBF0		LV
CBF1	CC0B	LVT
CC0C		LV
CC0D	CC27	LVT
CC28		LV
CC29	CC43	LVT
CC44		LV
CC45	CC5F	LVT
CC60		LV
CC61	CC7B	LVT
CC7C		LV
CC7D	CC97	LVT
CC98		LV
CC99	CCB3	LVT
CCB4		LV
CCB5	CCCF	LVT
CCD0		LV
CCD1	CCEB	LVT
CCEC		LV
CCED	CD07	LVT
CD08		LV
CD09	CD23	LVT
CD24		LV
CD25	CD3F	LVT
CD40		LV
CD41	CD5B	LVT
CD5C		LV
CD5D	CD77	LVT
CD78		LV
CD79	CD93	LVT
CD94		LV
CD95	CDAF	LVT
CDB0		LV
CDB1	CDCB	LVT
CDCC		LV
CDCD	CDE7	LVT
CDE8		LV
CDE9	CE03	LVT
CE04		LV
CE05	CE1F	LVT
CE20		LV
CE21	CE3B	LVT
CE3C		LV
CE3D	CE57	LVT
CE58		LV
CE59	CE73	LVT
CE74		LV
CE75	CE8F	LVT
CE90		LV
CE91	CEAB	LVT
CEAC		LV
CEAD	CEC7	LVT
CEC8		LV
CEC9	CEE3	LVT
CEE4		LV
CEE5	CEFF	LVT
CF00		LV
CF01	CF1B	LVT
CF1C		LV
CF1D	CF37	LVT
CF38		LV
CF39	CF53	LVT
CF54		LV
CF55	CF6F	LVT
CF70		LV
CF71	CF8B	LVT
CF8C		LV
CF8D	CFA7	LVT
CFA8		LV
CFA9	CFC3	LVT
CFC4		LV
CFC5	CFDF	LVT
CFE0		LV
CFE1	CFFB	LVT
CFFC		LV
CFFD	D017	LVT
D018		LV
D019	D033	LVT
D034		LV
D035	D04F	LVT
D050		LV
D051	D06B	LVT
D06C		LV
D06D	D087	LVT
D088		LV
D089	D0A3	LVT
D0A4		LV
D0A5	D0BF	LVT
D0C0		LV
D0C1	D0DB	LVT
D0DC		LV
D0DD	D0F7	LVT
D0F8		LV
D0F9	D113	LVT
D114		LV
D115	D12F	LVT
D130		LV
D131	D14B	LVT
D14C		LV
D14D	D167	LVT
D168		LV
D169	D183	LVT
D184		LV
D185	D19F	LVT
D1A0		LV
D1A1	D1BB	LVT
D1BC		LV
D1BD	D1D7	LVT
D1D8		LV
D1D9	D1F3	LVT
D1F4		LV
D1F5	D20F	LVT
D210		LV
D211	D22B	LVT
D22C		LV
D22D	D247	LVT
D248		LV
D249	D263	LVT
D264		LV
D265	D27F	LVT
D280		LV
D281	D29B	LVT
D29C		LV
D29D	D2B7	LVT
D2B8		LV
D2B9	D2D3	LVT
D2D4		LV
D2D5	D2EF	LVT
D2F0		LV
D2F1	D30B	LVT
D30C		LV
D30D	D327	LVT
D328		LV
D329	D343	LVT
D344		LV
D345	D35F	LVT
D360		LV
D361	D37B	LVT
D37C		LV
D37D	D397	LVT
D398		LV
D399	D3B3	LVT
D3B4		LV
D3B5	D3CF	LVT
D3D0		LV
D3D1	D3EB	LVT
D3EC		LV
D3ED	D407	LVT
D408		LV
D409	D423	LVT
D424		LV
D425	D43F	LVT
D440		LV
D441	D45B	LVT
D45C		LV
D45D	D477	LVT
D478		LV
D479	D493	LVT
D494		LV
D495	D4AF	LVT
D4B0		LV
D4B1	D4CB	LVT
D4CC		LV
D4CD	D4E7	LVT
D4E8		LV
D4E9	D503	LVT
D504		LV
D505	D51F	LVT
D520		LV
D521	D53B	LVT
D53C		LV
D53D	D557	LVT
D558		LV
D559	D573	LVT
D574		LV
D575	D58F	LVT
D590		LV
D591	D5AB	LVT
D5AC		LV
D5AD	D5C7	LVT
D5C8		LV
D5C9	D5E3	LVT
D5E4		LV
D5E5	D5FF	LVT
D600		LV
D601	D61B	LVT
D61C		LV
D61D	D637	LVT
D638		LV
D639	D653	LVT
D654		LV
D655	D66F	LVT
D670		LV
D671	D68B	LVT
D68C		LV
D68D	D6A7	LVT
D6A8		LV
D6A9	D6C3	LVT
D6C4		LV
D6C5	D6DF	LVT
D6E0		LV
D6E1	D6FB	LVT
D6FC		LV
D6FD	D717	LVT
D718		LV
D719	D733	LVT
D734		LV
D735	D74F	LVT
D750		LV
D751	D76B	LVT
D76C		LV
D76D	D787	LVT
D788		LV
D789	D7A3	LVT
D7B0	D7C6	V
D7CB	D7FB	T
FB1E		Extend
FE00	FE0F	Extend
FE20	FE2F	Extend
FEFF		Control
FF9E	FF9F	Extend
FFF0	FFFB	Control
101FD		Extend
102E0		Extend
10376	1037A	Extend
10A01	10A03	Extend
10A05	10A06	Extend
10A0C	10A0F	Extend
10A38	10A3A	Extend
10A3F		Extend
10AE5	10AE6	Extend
10D24	10D27	Extend
10EAB	10EAC	Extend
10EFD	10EFF	Extend
10F46	10F50	Extend
10F82	10F85	Extend
11000		SpacingMark
11001		Extend
11002		SpacingMark
11038	11046	Extend
11070		Extend
11073	11074	Extend
1107F	11081	Extend
11082		SpacingMark
110B0	110B2	SpacingMark
110B3	110B6	Extend
110B7	110B8	SpacingMark
110B9	110BA	Extend
110BD		Prepend
110C2		Extend
110CD		Prepend
11100	11102	Extend
11127	1112B	Extend
1112C		SpacingMark
1112D	11134	Extend
11145	11146	SpacingMark
11173		Extend
11180	11181	Extend
11182		SpacingMark
111B3	111B5	SpacingMark
111B6	111BE	Extend
111BF	111C0	SpacingMark
111C2	111C3	Prepend
111C9	111CC	Extend
111CE		SpacingMark
111CF		Extend
1122C	1122E	SpacingMark
1122F	11231	Extend
11232	11233	SpacingMark
11234		Extend
11235		SpacingMark
11236	11237	Extend
1123E		Extend
11241		Extend
112DF		Extend
112E0	112E2	SpacingMark
112E3	112EA	Extend
11300	11301	Extend
11302	11303	SpacingMark
1133B	1133C	Extend
1133E		Extend
1133F		SpacingMark
11340		Extend
11341	11344	SpacingMark
11347	11348	SpacingMark
1134B	1134D	SpacingMark
11357		Extend
11362	11363	SpacingMark
11366	1136C	Extend
11370	11374	Extend
11435	11437	SpacingMark
11438	1143F	Extend
11440	11441	SpacingMark
11442	11444	Extend
11445		SpacingMark
11446		Extend
1145E		Extend
114B0		Extend
114B1	114B2	SpacingMark
114B3	114B8	Extend
114B9		SpacingMark
114BA		Extend
114BB	114BC	SpacingMark
114BD		Extend
114BE		SpacingMark
114BF	114C0	Extend
114C1		SpacingMark
114C2	114C3	Extend
115AF		Extend
115B0	115B1	SpacingMark
115B2	115B5	Extend
115B8	115BB	SpacingMark
115BC	115BD	Extend
115BE		SpacingMark
115BF	115C0	Extend
115DC	115DD	Extend
11630	11632	SpacingMark
11633	1163A	Extend
1163B	1163C	SpacingMark
1163D		Extend
1163E		SpacingMark
1163F	11640	Extend
116AB		Extend
116AC		SpacingMark
116AD		Extend
116AE	116AF	SpacingMark
116B0	116B5	Extend
116B6		SpacingMark
116B7		Extend
1171D	1171F	Extend
11722	11725	Extend
11726		SpacingMark
11727	1172B	Extend
1182C	1182E	SpacingMark
1182F	11837	Extend
11838		SpacingMark
11839	1183A	Extend
11930		Extend
11931	11935	SpacingMark
11937	11938	SpacingMark
1193B	1193C	Extend
1193D		SpacingMark
1193E		Extend
1193F		Prepend
11940		SpacingMark
11941		Prepend
11942		SpacingMark
11943		Extend
119D1	119D3	SpacingMark
119D4	119D7	Extend
119DA	119DB	Extend
119DC	119DF	SpacingMark
119E0		Extend
119E4		SpacingMark
11A01	11A0A	Extend
11A33	11A38	Extend
11A39		SpacingMark
11A3A		Prepend
11A3B	11A3E	Extend
11A47		Extend
11A51	11A56	Extend
11A57	11A58	SpacingMark
11A59	11A5B	Extend
11A84	11A89	Prepend
11A8A	11A96	Extend
11A97		SpacingMark
11A98	11A99	Extend
11C2F		SpacingMark
11C30	11C36	Extend
11C38	11C3D	Extend
11C3E		SpacingMark
11C3F		Extend
11C92	11CA7	Extend
11CA9		SpacingMark
11CAA	11CB0	Extend
11CB1		SpacingMark
11CB2	11CB3	Extend
11CB4		SpacingMark
11CB5	11CB6	Extend
11D31	11D36	Extend
11D3A		Extend
11D3C	11D3D	Extend
11D3F	11D45	Extend
11D46		Prepend
11D47		Extend
11D8A	11D8E	SpacingMark
11D90	11D91	Extend
11D93	11D94	SpacingMark
11D95		Extend
11D96		SpacingMark
11D97		Extend
11EF3	11EF4	Extend
11EF5	11EF6	SpacingMark
11F00	11F01	Extend
11F02		Prepend
11F03		SpacingMark
11F34	11F35	SpacingMark
11F36	11F3A	Extend
11F3E	11F3F	SpacingMark
11F40		Extend
11F41		SpacingMark
11F42		Extend
13430	1343F	Control
13440		Extend
13447	13455	Extend
16AF0	16AF4	Extend
16B30	16B36	Extend
16F4F		Extend
16F51	16F87	SpacingMark
16F8F	16F92	Extend
16FE4		Extend
16FF0	16FF1	SpacingMark
1BC9D	1BC9E	Extend
1BCA0	1BCA3	Control
1CF00	1CF2D	Extend
1CF30	1CF46	Extend
1D165		Extend
1D166		SpacingMark
1D167	1D169	Extend
1D16D		SpacingMark
1D16E	1D172	Extend
1D173	1D17A	Control
1D17B	1D182	Extend
1D185	1D18B	Extend
1D1AA	1D1AD	Extend
1D242	1D244	Extend
1DA00	1DA36	Extend
1DA3B	1DA6C	Extend
1DA75		Extend
1DA84		Extend
1DA9B	1DA9F	Extend
1DAA1	1DAAF	Extend
1E000	1E006	Extend
1E008	1E018	Extend
1E01B	1E021	Extend
1E023	1E024	Extend
1E026	1E02A	Extend
1E08F		Extend
1E130	1E136	Extend
1E2AE		Extend
1E2EC	1E2EF	Extend
1E4EC	1E4EF	Extend
1E8D0	1E8D6	Extend
1E944	1E94A	Extend
1F000	1F0FF	ExtPict_XX
1F10D	1F10F	ExtPict_XX
1F12F		ExtPict_XX
1F16C	1F171	ExtPict_XX
1F17E	1F17F	ExtPict_XX
1F18E		ExtPict_XX
1F191	1F19A	ExtPict_XX
1F1AD	1F1E5	ExtPict_XX
1F1E6	1F1FF	Regional_Indicator
1F201	1F20F	ExtPict_XX
1F21A		ExtPict_XX
1F22F		ExtPict_XX
1F232	1F23A	ExtPict_XX
1F23C	1F23F	ExtPict_XX
1F249	1F3FA	ExtPict_XX
1F3FB	1F3FF	Extend
1F400	1F53D	ExtPict_XX
1F546	1F64F	ExtPict_XX
1F680	1F6FF	ExtPict_XX
1F774	1F77F	ExtPict_XX
1F7D5	1F7FF	ExtPict_XX
1F80C	1F80F	ExtPict_XX
1F848	1F84F	ExtPict_XX
1F85A	1F85F	ExtPict_XX
1F888	1F88F	ExtPict_XX
1F8AE	1F8FF	ExtPict_XX
1F90C	1F93A	ExtPict_XX
1F93C	1F945	ExtPict_XX
1F947	1FAFF	ExtPict_XX
1FC00	1FFFD	ExtPict_XX
E0000	E001F	Control
E0020	E007F	Extend
E0080	E00FF	Control
E0100	E01EF	Extend
E01F0	E0FFF	Control
END
