# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
# This file is machine-generated by lib/unicore/mktables from the Unicode
# database, Version 15.0.0.  Any changes made here will be lost!

# !!!!!!!   INTERNAL PERL USE ONLY   !!!!!!!
# This file is for internal use by core Perl only.  The format and even the
# name or existence of this file are subject to change without notice.  Don't
# use it directly.  Use Unicode::UCD to access the Unicode character data
# base.


return <<'END';
V38
0
10
11
13
14
32
127
160
173
174
1564
1565
6158
6159
8203
8204
8206
8208
8232
8239
8288
8304
65279
65280
65520
65532
78896
78912
113824
113828
119155
119163
917504
917536
917632
917760
918000
921600
END
