# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
# This file is machine-generated by lib/unicore/mktables from the Unicode
# database, Version 15.0.0.  Any changes made here will be lost!

# !!!!!!!   INTERNAL PERL USE ONLY   !!!!!!!
# This file is for internal use by core Perl only.  The format and even the
# name or existence of this file are subject to change without notice.  Don't
# use it directly.  Use Unicode::UCD to access the Unicode character data
# base.


return <<'END';
V52
2364
2365
2492
2493
2620
2621
2748
2749
2876
2877
3132
3133
3260
3261
4151
4152
6964
6965
7142
7143
7223
7224
43443
43444
69818
69819
70003
70004
70090
70091
70198
70199
70377
70378
70459
70461
70726
70727
70851
70852
71104
71105
71351
71352
71738
71739
72003
72004
73026
73027
125258
125259
END
