# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
# This file is machine-generated by lib/unicore/mktables from the Unicode
# database, Version 15.0.0.  Any changes made here will be lost!

# !!!!!!!   INTERNAL PERL USE ONLY   !!!!!!!
# This file is for internal use by core Perl only.  The format and even the
# name or existence of this file are subject to change without notice.  Don't
# use it directly.  Use Unicode::UCD to access the Unicode character data
# base.


return <<'END';
V30
1536
1542
1757
1758
1807
1808
2192
2194
2274
2275
3406
3407
69821
69822
69837
69838
70082
70084
71999
72000
72001
72002
72250
72251
72324
72330
73030
73031
73474
73475
END
