# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
# This file is machine-generated by lib/unicore/mktables from the Unicode
# database, Version 15.0.0.  Any changes made here will be lost!

# !!!!!!!   INTERNAL PERL USE ONLY   !!!!!!!
# This file is for internal use by core Perl only.  The format and even the
# name or existence of this file are subject to change without notice.  Don't
# use it directly.  Use Unicode::UCD to access the Unicode character data
# base.


return <<'END';
V58
1544
1545
1547
1548
1549
1550
1563
1611
1645
1648
1649
1750
1765
1767
1774
1776
1786
1809
1810
1840
1867
1958
1969
1984
2144
2192
2194
2200
2208
2250
64336
64830
64848
64975
65008
65021
65136
65279
68864
68900
68904
68912
68922
68928
69312
69373
69424
69446
69457
69488
126064
126144
126208
126288
126464
126704
126706
126720
END
