# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
# This file is machine-generated by lib/unicore/mktables from the Unicode
# database, Version 15.0.0.  Any changes made here will be lost!

# !!!!!!!   INTERNAL PERL USE ONLY   !!!!!!!
# This file is for internal use by core Perl only.  The format and even the
# name or existence of this file are subject to change without notice.  Don't
# use it directly.  Use Unicode::UCD to access the Unicode character data
# base.


return <<'END';
V40
768
834
835
837
838
867
2387
2389
6832
6863
7618
7672
7673
7674
7675
7680
8204
8206
8400
8432
65024
65040
65056
65070
66045
66046
118528
118574
118576
118599
119143
119146
119163
119171
119173
119180
119210
119214
917760
918000
END
