# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
# This file is machine-generated by lib/unicore/mktables from the Unicode
# database, Version 15.0.0.  Any changes made here will be lost!

# !!!!!!!   INTERNAL PERL USE ONLY   !!!!!!!
# This file is for internal use by core Perl only.  The format and even the
# name or existence of this file are subject to change without notice.  Don't
# use it directly.  Use Unicode::UCD to access the Unicode character data
# base.


return <<'END';
V36
64976
65008
65534
65536
131070
131072
196606
196608
262142
262144
327678
327680
393214
393216
458750
458752
524286
524288
589822
589824
655358
655360
720894
720896
786430
786432
851966
851968
917502
917504
983038
983040
1048574
1048576
1114110
1114112
END
