# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
# This file is machine-generated by lib/unicore/mktables from the Unicode
# database, Version 15.0.0.  Any changes made here will be lost!

# !!!!!!!   INTERNAL PERL USE ONLY   !!!!!!!
# This file is for internal use by core Perl only.  The format and even the
# name or existence of this file are subject to change without notice.  Don't
# use it directly.  Use Unicode::UCD to access the Unicode character data
# base.


return <<'END';
V56
33
48
58
65
91
95
96
97
123
127
161
168
169
170
171
173
174
175
176
178
182
183
187
188
191
192
215
216
247
248
8208
8232
8240
8255
8257
8276
8277
8287
8592
9312
9472
10102
10132
11264
11776
11904
12289
12292
12296
12321
12336
12337
64830
64832
65093
65095
END
