/* This is a placeholder file for symbols that should be exported
 * into config_h.SH and Porting/Glossary. See also metaconfig.SH
 *
 * First version was created from the part in handy.h
 * H.Merijn Brand 21 Dec 2010 (Tux)
 *
 * Mentioned variables are forced to be included into config_h.SH
 * as they are only included if meta finds them referenced. That
 * implies that noone can use them unless they are available and
 * they won't be available unless used. When new symbols are probed
 * in Configure, this is the way to force them into availability.
 *
 * Symbols should only be here temporarily. Once they are actually used,
 * they should be removed from here.
 *
 * BIN
 * CASTI32
 * CASTNEGFLOAT
 * CPPLAST
 * CPPMINUS
 * CPPRUN
 * CPPSTDIN
 * DOSUID
 * DOUBLE_HAS_NEGATIVE_ZERO
 * DOUBLE_HAS_SUBNORMALS
 * DOUBLEMANTBITS
 * DOUBLE_STYLE_CRAY
 * DOUBLE_STYLE_IBM
 * DOUBLE_STYLE_IEEE
 * DOUBLE_STYLE_VAX
 * DRAND48_R_PROTO
 * Gid_t_f
 * HAS_ASCTIME64
 * HAS_ATOLF
 * HAS_BUILTIN_ADD_OVERFLOW
 * HAS_BUILTIN_MUL_OVERFLOW
 * HAS_BUILTIN_SUB_OVERFLOW
 * HAS_CSH
 * HAS_CTERMID
 * HAS_CTIME64
 * HAS_DIFFTIME64
 * HAS_DRAND48_PROTO
 * HAS_DRAND48_R
 * HAS_FD_SET
 * HAS_FFS
 * HAS_FFSL
 * HAS_GETMNT
 * HAS_GMTIME64
 * HAS_GNULIBC
 * HAS_INT64_T
 * HAS_IPV6_MREQ_SOURCE
 * HAS_ISLESS
 * HAS_ISNORMAL
 * HAS_LGAMMA_R
 * HAS_LOCALECONV_L
 * HAS_LOCALTIME64
 * HAS_LSEEK_PROTO
 * HAS_MKTIME64
 * HAS_NANOSLEEP
 * HAS_OPEN3
 * HAS_OPENAT
 * HAS_PRCTL
 * HAS_PSEUDOFORK
 * HAS_RANDOM_R
 * HAS_SIGINFO_SI_VALUE
 * HAS_SIGSETJMP
 * HAS_SRAND48_R
 * HAS_SRANDOM_R
 * HAS_STRTOD_L
 * HAS_STRTOLD_L
 * HAS_STRUCT_FS_DATA
 * HAS_STRUCT_STATFS
 * HAS_STRUCT_STATFS_F_FLAGS
 * HAS_TIME
 * HAS_USTAT
 * HAS_VFORK
 * HAS_WCSCMP
 * HAS_WCSXFRM
 * I16SIZE
 * I32SIZE
 * I64SIZE
 * I8SIZE
 * I_GDBM
 * INSTALL_USR_BIN_PERL
 * I_STDBOOL
 * I_SYS_MOUNT
 * I_SYS_STATFS
 * I_SYS_STATVFS
 * I_SYS_VFS
 * I_TIME
 * I_USTAT
 * I_VFORK
 * I_XLOCALE
 * LOCALTIME_R_NEEDS_TZSET
 * LOC_SED
 * LONGDBLMANTBITS
 * LONG_DOUBLE_STYLE_IEEE
 * LONG_DOUBLE_STYLE_IEEE_DOUBLEDOUBLE
 * LONG_DOUBLE_STYLE_IEEE_EXTENDED
 * LONG_DOUBLE_STYLE_IEEE_STD
 * LONG_DOUBLE_STYLE_VAX
 * OSVERS
 * PERL_LC_ALL_CATEGORY_POSITIONS_INIT
 * PERL_LC_ALL_SEPARATOR
 * PERL_LC_ALL_USES_NAME_VALUE_PAIRS
 * PERL_PRIeldbl
 * PERL_SCNfldbl
 * PERL_TARGETARCH
 * PERL_VENDORARCH
 * RANDOM_R_PROTO
 * SRAND48_R_PROTO
 * SRANDOM_R_PROTO
 * STARTPERL
 * ST_INO_SIGN
 * ST_INO_SIZE
 * U32_ALIGNMENT_REQUIRED
 * U32of
 * U32xf
 * U32Xf
 * U8SIZE
 * Uid_t_f
 * USE_CROSS_COMPILE
 * USE_MORE_BITS
 */
