/* -*- mode: C; buffer-read-only: t -*-
 *
 *    keywords.h
 *
 *    Copyright (C) 1994, 1995, 1996, 1997, 1999, 2000, 2001, 2002, 2005,
 *    2006, 2007 by Larry Wall and others
 *
 *    You may distribute under the terms of either the GNU General Public
 *    License or the Artistic License, as specified in the README file.
 *
 * !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
 * This file is built by regen/keywords.pl from its data.
 * Any changes made here will be lost!
 */

#define KEY_NULL		0
#define KEY___FILE__		1
#define KEY___LINE__		2
#define KEY___PACKAGE__		3
#define KEY___CLASS__		4
#define KEY___DATA__		5
#define KEY___END__		6
#define KEY___SUB__		7
#define KEY_ADJUST		8
#define KEY_AUTOLOAD		9
#define KEY_BEGIN		10
#define KEY_UNITCHECK		11
#define KEY_DESTROY		12
#define KEY_END			13
#define KEY_INIT		14
#define KEY_CHECK		15
#define KEY_abs			16
#define KEY_accept		17
#define KEY_alarm		18
#define KEY_and			19
#define KEY_atan2		20
#define KEY_bind		21
#define KEY_binmode		22
#define KEY_bless		23
#define KEY_break		24
#define KEY_caller		25
#define KEY_catch		26
#define KEY_chdir		27
#define KEY_chmod		28
#define KEY_chomp		29
#define KEY_chop		30
#define KEY_chown		31
#define KEY_chr			32
#define KEY_chroot		33
#define KEY_class		34
#define KEY_close		35
#define KEY_closedir		36
#define KEY_cmp			37
#define KEY_connect		38
#define KEY_continue		39
#define KEY_cos			40
#define KEY_crypt		41
#define KEY_dbmclose		42
#define KEY_dbmopen		43
#define KEY_default		44
#define KEY_defer		45
#define KEY_defined		46
#define KEY_delete		47
#define KEY_die			48
#define KEY_do			49
#define KEY_dump		50
#define KEY_each		51
#define KEY_else		52
#define KEY_elsif		53
#define KEY_endgrent		54
#define KEY_endhostent		55
#define KEY_endnetent		56
#define KEY_endprotoent		57
#define KEY_endpwent		58
#define KEY_endservent		59
#define KEY_eof			60
#define KEY_eq			61
#define KEY_eval		62
#define KEY_evalbytes		63
#define KEY_exec		64
#define KEY_exists		65
#define KEY_exit		66
#define KEY_exp			67
#define KEY_fc			68
#define KEY_fcntl		69
#define KEY_field		70
#define KEY_fileno		71
#define KEY_finally		72
#define KEY_flock		73
#define KEY_for			74
#define KEY_foreach		75
#define KEY_fork		76
#define KEY_format		77
#define KEY_formline		78
#define KEY_ge			79
#define KEY_getc		80
#define KEY_getgrent		81
#define KEY_getgrgid		82
#define KEY_getgrnam		83
#define KEY_gethostbyaddr	84
#define KEY_gethostbyname	85
#define KEY_gethostent		86
#define KEY_getlogin		87
#define KEY_getnetbyaddr	88
#define KEY_getnetbyname	89
#define KEY_getnetent		90
#define KEY_getpeername		91
#define KEY_getpgrp		92
#define KEY_getppid		93
#define KEY_getpriority		94
#define KEY_getprotobyname	95
#define KEY_getprotobynumber	96
#define KEY_getprotoent		97
#define KEY_getpwent		98
#define KEY_getpwnam		99
#define KEY_getpwuid		100
#define KEY_getservbyname	101
#define KEY_getservbyport	102
#define KEY_getservent		103
#define KEY_getsockname		104
#define KEY_getsockopt		105
#define KEY_given		106
#define KEY_glob		107
#define KEY_gmtime		108
#define KEY_goto		109
#define KEY_grep		110
#define KEY_gt			111
#define KEY_hex			112
#define KEY_if			113
#define KEY_index		114
#define KEY_int			115
#define KEY_ioctl		116
#define KEY_isa			117
#define KEY_join		118
#define KEY_keys		119
#define KEY_kill		120
#define KEY_last		121
#define KEY_lc			122
#define KEY_lcfirst		123
#define KEY_le			124
#define KEY_length		125
#define KEY_link		126
#define KEY_listen		127
#define KEY_local		128
#define KEY_localtime		129
#define KEY_lock		130
#define KEY_log			131
#define KEY_lstat		132
#define KEY_lt			133
#define KEY_m			134
#define KEY_map			135
#define KEY_method		136
#define KEY_mkdir		137
#define KEY_msgctl		138
#define KEY_msgget		139
#define KEY_msgrcv		140
#define KEY_msgsnd		141
#define KEY_my			142
#define KEY_ne			143
#define KEY_next		144
#define KEY_no			145
#define KEY_not			146
#define KEY_oct			147
#define KEY_open		148
#define KEY_opendir		149
#define KEY_or			150
#define KEY_ord			151
#define KEY_our			152
#define KEY_pack		153
#define KEY_package		154
#define KEY_pipe		155
#define KEY_pop			156
#define KEY_pos			157
#define KEY_print		158
#define KEY_printf		159
#define KEY_prototype		160
#define KEY_push		161
#define KEY_q			162
#define KEY_qq			163
#define KEY_qr			164
#define KEY_quotemeta		165
#define KEY_qw			166
#define KEY_qx			167
#define KEY_rand		168
#define KEY_read		169
#define KEY_readdir		170
#define KEY_readline		171
#define KEY_readlink		172
#define KEY_readpipe		173
#define KEY_recv		174
#define KEY_redo		175
#define KEY_ref			176
#define KEY_rename		177
#define KEY_require		178
#define KEY_reset		179
#define KEY_return		180
#define KEY_reverse		181
#define KEY_rewinddir		182
#define KEY_rindex		183
#define KEY_rmdir		184
#define KEY_s			185
#define KEY_say			186
#define KEY_scalar		187
#define KEY_seek		188
#define KEY_seekdir		189
#define KEY_select		190
#define KEY_semctl		191
#define KEY_semget		192
#define KEY_semop		193
#define KEY_send		194
#define KEY_setgrent		195
#define KEY_sethostent		196
#define KEY_setnetent		197
#define KEY_setpgrp		198
#define KEY_setpriority		199
#define KEY_setprotoent		200
#define KEY_setpwent		201
#define KEY_setservent		202
#define KEY_setsockopt		203
#define KEY_shift		204
#define KEY_shmctl		205
#define KEY_shmget		206
#define KEY_shmread		207
#define KEY_shmwrite		208
#define KEY_shutdown		209
#define KEY_sin			210
#define KEY_sleep		211
#define KEY_socket		212
#define KEY_socketpair		213
#define KEY_sort		214
#define KEY_splice		215
#define KEY_split		216
#define KEY_sprintf		217
#define KEY_sqrt		218
#define KEY_srand		219
#define KEY_stat		220
#define KEY_state		221
#define KEY_study		222
#define KEY_sub			223
#define KEY_substr		224
#define KEY_symlink		225
#define KEY_syscall		226
#define KEY_sysopen		227
#define KEY_sysread		228
#define KEY_sysseek		229
#define KEY_system		230
#define KEY_syswrite		231
#define KEY_tell		232
#define KEY_telldir		233
#define KEY_tie			234
#define KEY_tied		235
#define KEY_time		236
#define KEY_times		237
#define KEY_tr			238
#define KEY_try			239
#define KEY_truncate		240
#define KEY_uc			241
#define KEY_ucfirst		242
#define KEY_umask		243
#define KEY_undef		244
#define KEY_unless		245
#define KEY_unlink		246
#define KEY_unpack		247
#define KEY_unshift		248
#define KEY_untie		249
#define KEY_until		250
#define KEY_use			251
#define KEY_utime		252
#define KEY_values		253
#define KEY_vec			254
#define KEY_wait		255
#define KEY_waitpid		256
#define KEY_wantarray		257
#define KEY_warn		258
#define KEY_when		259
#define KEY_while		260
#define KEY_write		261
#define KEY_x			262
#define KEY_xor			263
#define KEY_y			264

/* Generated from:
 * c8b75109fa56ce3ea3f30503a3b398f02e49036dc60d5fb36ea5ba9ffd6c596e regen/keywords.pl
 * ex: set ro ft=c: */
